<?php

namespace FiloBlu\ExtTargetRule\Plugin;

use Magento\TargetRule\Model\Index;

class TargetRuleExtensionPlugin extends \Magento\TargetRule\Model\ResourceModel\Index
{
    /**
     * @param Index $object
     * @return array
     */
    public function getProductIds($object)
    {
        $originalLimit = $object->getLimit();

        $limit = (int)$originalLimit;

        $ruleCollection = $object->getRuleCollection();
        foreach ($ruleCollection as $rule) {
            $limit = min($limit, (int)$rule->getPositionsLimit());
        }

        $object->setLimit(10000);

        $result = parent::getProductIds($object);

        $object->setLimit($originalLimit);

        return array_slice($result, 0, $limit);
    }

    /**
     * Match, save and return applicable product ids by segmentId object
     *
     * @param \Magento\TargetRule\Model\Index $object
     * @param string $segmentId
     * @return array
     */
    protected function _matchProductIdsBySegmentId($object, $segmentId)
    {
        $limit = $object->getLimit() + $this->getOverfillLimit();
        $productIds = [];
        $ruleCollection = $object->getRuleCollection();
        if ($this->_customerSegmentData->isEnabled()) {
            $ruleCollection->addSegmentFilter($segmentId);
        }
        foreach ($ruleCollection as $rule) {
            /* @var $rule \Magento\TargetRule\Model\Rule */
            if (count($productIds) >= $limit) {
                break;
            }
            if (!$rule->checkDateForStore($object->getStoreId())) {
                continue;
            }
            $excludeProductIds = array_merge([$object->getProduct()->getEntityId()], $productIds);
            $resultIds = $this->_getProductIdsByRule($rule, $object, $limit, $excludeProductIds);
            $productIds = array_merge($productIds, $resultIds);
        }
        return $productIds;
    }
}